﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Syncfusion.EJ2.Base;
using Syncfusion.EJ2.Linq;
using AngularwithASPCore.Models;
using Newtonsoft.Json;
using TestSample.Models;

namespace TestSample.Controllers
{

    public class HomeController : Controller
    {

        public IActionResult Index()
        {
            ViewBag.datasource = OrdersDetails.GetAllRecords().Take(20).ToList();
           return View();
        }

        public IActionResult GridDatasource([FromBody]DataManagerRequest dm)
        {
            IEnumerable<OrdersDetails> DataSource = OrdersDetails.GetAllRecords();

            DataOperations operation = new DataOperations();
            if (dm.Search != null && dm.Search.Count > 0)
            {
                DataSource = operation.PerformSearching(DataSource, dm.Search);  //Search
            }
            if (dm.Sorted != null && dm.Sorted.Count > 0) //Sorting
            {
                DataSource = operation.PerformSorting(DataSource, dm.Sorted);
            }
            if (dm.Where != null && dm.Where.Count > 0) //Filtering
            {
                DataSource = operation.PerformFiltering(DataSource, dm.Where, dm.Where[0].Operator);
            }
            int count = DataSource.Cast<OrdersDetails>().Count();
            if (dm.Skip != 0)
            {
                DataSource = operation.PerformSkip(DataSource, dm.Skip);   //Paging
            }
            if (dm.Take != 0)
            {
                DataSource = operation.PerformTake(DataSource, dm.Take);
            }
            //return Json(new { result = DataSource, count = count });
            return dm.RequiresCounts ? Json(new { result = DataSource, count = count }) : Json(DataSource);
        }
        public ActionResult Update([FromBody]CRUDModel<OrdersDetails> value)
        {
            var ord = value.Value;
            OrdersDetails val = OrdersDetails.GetAllRecords().Where(or => or.OrderID == ord.OrderID).FirstOrDefault();
            val.OrderID = ord.OrderID;
            val.EmployeeID = ord.EmployeeID;
            val.CustomerID = ord.CustomerID;
            val.Freight = ord.Freight;
            val.OrderDate = ord.OrderDate;
            val.ShipCity = ord.ShipCity;

            return Json(value.Value);
        }
        //insert the record
        public ActionResult Insert([FromBody]CRUDModel<OrdersDetails> value)
        {
            if (value.Value.OrderID < 10000)
                throw new Exception("OrderID must be above 10000");
            OrdersDetails.GetAllRecords().Insert(0, value.Value);
            return Json(value.Value);
        }
        //Delete the record
        public ActionResult Delete([FromBody]CRUDModel<OrdersDetails> value)
        {
            OrdersDetails.GetAllRecords().Remove(OrdersDetails.GetAllRecords().Where(or => or.OrderID == int.Parse(value.Key.ToString())).FirstOrDefault());
            return Json(value);
        }

        ////Multiple Delete record - batch update
        //public IActionResult BatchUpdate([FromBody]CRUDModel value)
        //{
        //    if (value.Changed != null)
        //    {
        //        for (var i = 0; i < value.Changed.Count(); i++)
        //        {
        //            var ord = value.Changed[i];
        //            OrdersDetails val = OrdersDetails.GetAllRecords().Where(or => or.OrderID == ord.OrderID).FirstOrDefault();
        //            val.OrderID = ord.OrderID;
        //            val.EmployeeID = ord.EmployeeID;
        //            val.CustomerID = ord.CustomerID;
        //            val.Freight = ord.Freight;
        //            val.OrderDate = ord.OrderDate;
        //            val.ShipCity = ord.ShipCity;
        //        }
        //    }

        //    if (value.Deleted != null)
        //    {
        //        for (var i = 0; i < value.Deleted.Count(); i++)
        //        {
        //            OrdersDetails.GetAllRecords().Remove(OrdersDetails.GetAllRecords().Where(or => or.OrderID == value.Deleted[i].OrderID).FirstOrDefault());
        //        }
        //    }

        //    if (value.Added != null)
        //    {
        //        for (var i = 0; i < value.Added.Count(); i++)
        //        {
        //            OrdersDetails.GetAllRecords().Insert(0, value.Added[i]);
        //        }
        //    }
        //    var data = OrdersDetails.GetAllRecords().ToList();
        //    return Json(data);

        //}
        public class Data
        {

            public bool requiresCounts { get; set; }
            public int skip { get; set; }
            public int take { get; set; }
        }
        //public class CRUDModel
        //{
        //    public List<OrdersDetails> Added { get; set; }
        //    public List<OrdersDetails> Changed { get; set; }
        //    public List<OrdersDetails> Deleted { get; set; }
        //    public OrdersDetails Value { get; set; }
        //    public int key { get; set; }
        //    public string action { get; set; }
        //}
    }
}
